@@ -1,24 +1,45 @@
+# Copyright 2020-     Robot Framework Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import json
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, List, Optional, Union
 
 from robotlibcore import keyword  # type: ignore
 
 from ..assertion_engine import (
     bool_verify_assertion,
     dict_verify_assertion,
+    float_str_verify_assertion,
     int_dict_verify_assertion,
-    int_str_verify_assertion,
     list_verify_assertion,
     verify_assertion,
+    with_assertion_polling,
 )
 from ..base import LibraryComponent
 from ..generated.playwright_pb2 import Request
 from ..utils import logger
-from ..utils.data_types import AssertionOperator, BoundingBoxFields, SelectAttribute
+from ..utils.data_types import (
+    AssertionOperator,
+    BoundingBoxFields,
+    SelectAttribute,
+    ViewportFields,
+)
 
 
 class Getters(LibraryComponent):
-    @keyword(tags=["Getter", "Assertion", "BrowserControl"])
+    @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_url(
         self,
         assertion_operator: Optional[AssertionOperator] = None,
@@ -38,13 +59,15 @@ class Getters(LibraryComponent):
                 value, assertion_operator, assertion_expected, "URL "
             )
 
-    @keyword(tags=["Getter", "Assertion", "BrowserControl"])
+    # @keyword(tags=["Getter", "Assertion", "BrowserControl"])
+    # Not published as keyword due to missing of good docs.
+    @with_assertion_polling
     def get_page_state(
         self,
         assertion_operator: Optional[AssertionOperator] = None,
         assertion_expected: Any = None,
     ) -> object:
-        """Returns page model state object.
+        """Returns page model state object as a dictionary.
 
         See `Assertions` for further details for the assertion arguments. Defaults to None.
 
@@ -64,7 +87,8 @@ class Getters(LibraryComponent):
                 value, assertion_operator, assertion_expected, "State "
             )
 
-    @keyword(tags=["Getter", "BrowserControl"])
+    @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_page_source(
         self,
         assertion_operator: Optional[AssertionOperator] = None,
@@ -85,6 +109,7 @@ class Getters(LibraryComponent):
             )
 
     @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_title(
         self,
         assertion_operator: Optional[AssertionOperator] = None,
@@ -115,19 +140,52 @@ class Getters(LibraryComponent):
 
         Optionally asserts that the text matches the specified assertion.
 
+        See `Assertions` for further details for the assertion arguments. Defaults to None.
+        """
+        return self.get_property(
+            selector, "innerText", assertion_operator, assertion_expected
+        )
+
+    @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
+    def get_property(
+        self,
+        selector: str,
+        property: str,
+        assertion_operator: Optional[AssertionOperator] = None,
+        assertion_expected: Any = None,
+    ):
+        """Returns the ``property`` of the element found by ``selector``.
+
+        Optionally asserts that the property value matches the specified
+        assertion.
+
+        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
+
+        ``property`` <str> Requested property name. **Required**
+
+        If ``assertion_operator`` is set and property is not found, ``value`` is ``None``
+        and Keyword does not fail. See `Get Attribute` for examples.
+
         See `Assertions` for further details for the assertion arguments. Defaults to None.
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.GetDomProperty(
-                Request().ElementProperty(selector=selector, property="innerText")
+                Request().ElementProperty(selector=selector, property=property)
             )
             logger.debug(response.log)
-            value = response.body
+            if response.body:
+                value = json.loads(response.body)
+            elif assertion_operator is not None:
+                value = None
+            else:
+                raise AttributeError(f"Property '{property}' not found!")
             return verify_assertion(
-                value, assertion_operator, assertion_expected, f"Text {selector}"
+                value, assertion_operator, assertion_expected, f"Property {property}"
             )
 
     @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_attribute(
         self,
         selector: str,
@@ -135,7 +193,7 @@ class Getters(LibraryComponent):
         assertion_operator: Optional[AssertionOperator] = None,
         assertion_expected: Any = None,
     ):
-        """Returns the ``attribute`` of the element found by ``selector``.
+        """Returns the HTML ``attribute`` of the element found by ``selector``.
 
         Optionally asserts that the attribute value matches the specified
         assertion.
@@ -144,19 +202,99 @@ class Getters(LibraryComponent):
 
         ``attribute`` <str> Requested attribute name. **Required**
 
+        When a attribute is selected that is not present and no assertion operator is set,
+        the keyword fails. If an assertion operator is set and the attribute is not present,
+        the returned value is ``None``.
+        This can be used to assert check the presents or the absents of an attribute.
+
+        Example Element:
+        | <button class="login button active" id="enabled_button" something>Login</button>
+
+        Example Code:
+        | Get Attribute   id=enabled_button    disabled                   # FAIL => "Attribute 'disabled' not found!"
+        | Get Attribute   id=enabled_button    disabled     ==    None     # PASS => returns: None
+        | Get Attribute   id=enabled_button    something    evaluate    value is not None    # PASS =>  returns: True
+        | Get Attribute   id=enabled_button    disabled     evaluate    value is None        # PASS =>  returns: True
+
+
         See `Assertions` for further details for the assertion arguments. Defaults to None.
         """
         with self.playwright.grpc_channel() as stub:
-            response = stub.GetDomProperty(
+            response = stub.GetElementAttribute(
                 Request().ElementProperty(selector=selector, property=attribute)
             )
             logger.debug(response.log)
-            value = response.body
+            value = json.loads(response.body)
+            if assertion_operator is None and value is None:
+                raise AttributeError(f"Attribute '{attribute}' not found!")
+            logger.debug(f"Attribute is: {value}")
             return verify_assertion(
                 value, assertion_operator, assertion_expected, f"Attribute {selector}"
             )
 
     @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
+    def get_attribute_names(
+        self,
+        selector: str,
+        assertion_operator: Optional[AssertionOperator] = None,
+        *assertion_expected,
+    ):
+        """Returns all HTML attribute names of an element as a list.
+
+        Optionally asserts that these match the specified assertion.
+
+        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
+
+        ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.
+
+        Available assertions:
+        - ``==`` and ``!=`` can work with multiple values
+        - ``contains``/``*=`` only accepts one single expected value
+
+        Other operators are not allowed.
+        """
+        attribute_names = self.library.execute_javascript(
+            "(element) => element.getAttributeNames()", selector
+        )
+        expected = list(assertion_expected)
+        return list_verify_assertion(
+            attribute_names, assertion_operator, expected, "Attribute names"
+        )
+
+    @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
+    def get_classes(
+        self,
+        selector: str,
+        assertion_operator: Optional[AssertionOperator] = None,
+        *assertion_expected,
+    ):
+        """Returns all classes of an element as a list.
+
+        Optionally asserts that the value matches the specified assertion.
+
+        ``selector`` <str> Selector from which the info is to be retrieved. **Required**
+
+        ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.
+
+        Available assertions:
+        - ``==`` and ``!=`` can work with multiple values
+        - ``contains``/``*=`` only accepts one single expected value
+
+        Other operators are not allowed.
+        """
+        class_dict = self.get_property(selector, "classList")
+        expected = list(assertion_expected)
+        return list_verify_assertion(
+            list(class_dict.values()),
+            assertion_operator,
+            expected,
+            f"Classes of {selector}",
+        )
+
+    @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_textfield_value(
         self,
         selector: str,
@@ -171,11 +309,15 @@ class Getters(LibraryComponent):
 
         See `Assertions` for further details for the assertion arguments. Defaults to None.
         """
-        return self.get_attribute(
-            selector, "value", assertion_operator, assertion_expected
+        return verify_assertion(
+            self.get_property(selector, "value"),
+            assertion_operator,
+            assertion_expected,
+            f"Value {selector}",
         )
 
     @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_selected_options(
         self,
         selector: str,
@@ -190,7 +332,7 @@ class Getters(LibraryComponent):
         ``selector`` <str> Selector from which the info is to be retrieved. **Required**
 
         ``option_attribute`` <SelectAttribute.label> Which attribute shall be returned/verified.
-        Allowed values are ``<"value"|"label"|"text"|"index">``. Defaults to label.
+        Allowed values are ``< ``value`` | ``label`` | ``text`` | ``index`` >``. Defaults to label.
 
         ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.
 
@@ -201,13 +343,13 @@ class Getters(LibraryComponent):
 
         Example:
 
-        | `Select Options By`    | label                  | //select[2]  | Email    | Mobile           |      |                     |
-        | ${selected_list}       | `Get Selected Options` | //select[2]  |          |                  |      | # getter            |
-        | `Get Selected Options` | //select[2]            | label        | `==`     | Mobile           | Mail | #assertion content  |
-        | `Select Options By`    | label                  | select#names | 2        | 4                |      |                     |
-        | `Get Selected Options` | select#names           | index        | `==`     | 2                | 4    | #assertion index    |
-        | `Get Selected Options` | select#names           | label        | *=       | Mikko            |      | #assertion contains |
-        | `Get Selected Options` | select#names           | label        | validate | len(value) == 3  |      | #assertion length   |
+        | `Select Options By`      label                    //select[2]    Email      Mobile
+        | ${selected_list}         `Get Selected Options`   //select[2]                                         # getter
+        | `Get Selected Options`   //select[2]              label          `==`       Mobile             Mail   #assertion content
+        | `Select Options By`      label                    select#names   2          4
+        | `Get Selected Options`   select#names             index          `==`       2                  4      #assertion index
+        | `Get Selected Options`   select#names             label          *=         Mikko                     #assertion contain
+        | `Get Selected Options`   select#names             label          validate   len(value) == 3           #assertion length
 
         """
         with self.playwright.grpc_channel() as stub:
@@ -232,6 +374,7 @@ class Getters(LibraryComponent):
             )
 
     @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_checkbox_state(
         self,
         selector: str,
@@ -246,7 +389,7 @@ class Getters(LibraryComponent):
 
         ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.
 
-        - ``==`` and ``!=`` are allowed on boolean values
+        - ``==`` and ``!=`` and equivalent are allowed on boolean values
         - other operators are not accepted.
 
         ``expected_state`` <str> boolean value of expected state.
@@ -263,14 +406,15 @@ class Getters(LibraryComponent):
             response = stub.GetBoolProperty(
                 Request().ElementProperty(selector=selector, property="checked")
             )
-        logger.info(response.log)
-        value: bool = response.body
-        logger.info(f"Checkbox is {'checked' if value else 'unchecked'}")
-        return bool_verify_assertion(
-            value, assertion_operator, expected_state, f"Checkbox {selector} is"
-        )
+            logger.info(response.log)
+            value: bool = response.body
+            logger.info(f"Checkbox is {'checked' if value else 'unchecked'}")
+            return bool_verify_assertion(
+                value, assertion_operator, expected_state, f"Checkbox {selector} is"
+            )
 
     @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_element_count(
         self,
         selector: str,
@@ -286,15 +430,13 @@ class Getters(LibraryComponent):
         ``assertion_operator`` <AssertionOperator> See `Assertions` for further details. Defaults to None.
 
         ``expected_value`` <str|int> Expected value for the counting
-
-
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.GetElementCount(
                 Request().ElementSelector(selector=selector)
             )
             count = response.body
-            return int_str_verify_assertion(
+            return float_str_verify_assertion(
                 int(count),
                 assertion_operator,
                 expected_value,
@@ -302,73 +444,49 @@ class Getters(LibraryComponent):
             )
 
     @keyword(tags=["Getter", "Assertion", "BrowserControl"])
-    def get_browser_catalog(self):
-        """ Returns all browsers, open contexts in them and open pages in these contexts.
-
-            The data is parsed into a python list containing data representing the open Objects.
-
-            On the root level the data contains a list of open browsers.
-
-            Browser: ``{type: Literal['chromium', 'firefox', 'webkit'], 'id': int, contexts: List[Context]}``
-
-            Context: ``{type: 'context', 'id': int, pages: List[Page]}``
-
-            Page: ``{type: 'page', 'id': int, title: str, url: str}``
-
-            Sample:
-            | [{
-            |     "type": "firefox",
-            |     "id": 0,
-            |     "contexts": [{
-            |         "type": "context",
-            |         "id": 0,
-            |         "pages": [{
-            |             "type": "page",
-            |             "title": "prefilled_email_form.html",
-            |             "url": "http://localhost:7272/prefilled_email_form.html",
-            |             "id": "0"
-            |         }]
-            |     }, {
-            |         "type": "context",
-            |         "id": 1,
-            |         "pages": [{
-            |             "type": "page",
-            |             "title": "Login Page",
-            |             "url": "http://localhost:7272/dist/",
-            |             "id": "0"
-            |         }]
-            |     }]
-            | }]
-
-        """
-        with self.playwright.grpc_channel() as stub:
-            response = stub.GetBrowserCatalog(Request().Empty())
-            parsed = json.loads(response.body)
-            logger.info(json.dumps(parsed))
-            return parsed
-
-    @keyword(tags=["Getter", "Assertion", "BrowserControl"])
+    @with_assertion_polling
     def get_viewport_size(
         self,
+        key: ViewportFields = ViewportFields.ALL,
         assertion_operator: Optional[AssertionOperator] = None,
-        assertion_expected: Optional[Dict[str, int]] = None,
+        assertion_expected: Any = None,
     ):
         """Returns the current viewport dimensions.
 
         Optionally asserts that the count matches the specified assertion.
 
+        ``key`` < ``width`` | ``height`` | ``ALL`` > Optionally filters the returned values.
+        If keys is set to ``ALL``(default) it will return the viewport size as dictionary,
+        otherwise it will just return the single value selected by the key.
+        Note: If a single value is retrieved, an assertion does *not* need a ``validate``
+        combined with a cast of ``value``.
+
         See `Assertions` for further details for the assertion arguments. Defaults to None.
 
-        ``assertion_expected`` <Dict<str, int>> Defaults to None.
-         """
+        Example:
+        | Get Viewport Size    ALL    ==    {'width':1280, 'height':720}
+        | Get Viewport Size    width    >=    1200
+
+        """
         with self.playwright.grpc_channel() as stub:
             response = stub.GetViewportSize(Request().Empty())
-            parsed = json.loads(response.body)
-            return int_dict_verify_assertion(
-                parsed, assertion_operator, assertion_expected, "Viewport size is"
-            )
+            logger.info(response.log)
+            parsed = json.loads(response.json)
+            logger.debug(parsed)
+            if key == ViewportFields.ALL:
+                return int_dict_verify_assertion(
+                    parsed, assertion_operator, assertion_expected, "Viewport size is"
+                )
+            else:
+                logger.info(f"Value of '{key}'': {parsed[key.name]}")
+                return float_str_verify_assertion(
+                    parsed[key.name],
+                    assertion_operator,
+                    assertion_expected,
+                    f"{key} is ",
+                )
 
-    @keyword(tags=["Getter", "BrowserControl"])
+    @keyword(tags=["Getter", "PageContent"])
     def get_element(self, selector: str):
         """Returns a reference to a Playwright element handle.
 
@@ -381,7 +499,7 @@ class Getters(LibraryComponent):
             response = stub.GetElement(Request().ElementSelector(selector=selector))
             return response.body
 
-    @keyword(tags=["Getter", "BrowserControl"])
+    @keyword(tags=["Getter", "PageContent"])
     def get_elements(self, selector: str):
         """Returns a reference to playwright element handle for all matched elements by ``selector``.
 
@@ -389,10 +507,10 @@ class Getters(LibraryComponent):
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.GetElements(Request().ElementSelector(selector=selector))
-            data = json.loads(response.body)
-            return data
+            return json.loads(response.json)
 
-    @keyword(tags=["Getter", "Assertion"])
+    @keyword(tags=["Getter", "Assertion", "PageContent"])
+    @with_assertion_polling
     def get_style(
         self,
         selector: str,
@@ -412,7 +530,7 @@ class Getters(LibraryComponent):
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.GetStyle(Request().ElementSelector(selector=selector))
-            parsed = json.loads(response.body)
+            parsed = json.loads(response.json)
 
             if key == "ALL":
                 return dict_verify_assertion(
@@ -429,26 +547,48 @@ class Getters(LibraryComponent):
                     f"Style value for {key} is ",
                 )
 
-    @keyword(tags=["Getter", "Assertion"])
-    def get_boundingbox(self, selector: str, *keys: BoundingBoxFields):
-        """ Gets elements size and location as an object {x: int, y: int, width: int, height: int}.
+    @keyword(tags=["Getter", "Assertion", "PageContent"])
+    def get_boundingbox(
+        self,
+        selector: str,
+        key: BoundingBoxFields = BoundingBoxFields.ALL,
+        assertion_operator: Optional[AssertionOperator] = None,
+        assertion_expected: Any = None,
+    ):
+        """Gets elements size and location as an object {x: float, y: float, width: float, height: float}.
 
-            ``selector`` <str> Selector from which shall be retrieved. **Required**
+        ``selector`` <str> Selector from which shall be retrieved. **Required**
 
-            ``keys`` <BoundingBoxFields> Optionally filters the returned values.
-
-            Example use:
-            | unfiltered:       |                  |            |   |   |
-            | ${bounding_box}=  | Get BoundingBox  | \\#element |   |   |
-            | filtered:         |                  |            |   |   |
-            | ${xy}=            | Get BoundingBox  | \\#element | x | y |
+        ``key`` < ``x`` | ``y`` | ``width`` | ``height`` | ``ALL`` > Optionally filters the returned values.
+        If keys is set to ``ALL``(default) it will return the BoundingBox as Dictionary,
+        otherwise it will just return the single value selected by the key.
+        Note: If a single value is retrieved, an assertion does *not* need a ``validate``
+        combined with a cast of ``value``.
 
+        See `Assertions` for further details for the assertion arguments. Defaults to None.
 
+        Example use:
+        | ${bounding_box}=    Get BoundingBox    id=element                 # unfiltered
+        | Log                 ${bounding_box}                               # {'x': 559.09375, 'y': 75.5, 'width': 188.796875, 'height': 18}
+        | ${x}=               Get BoundingBox    id=element    x            # filtered
+        | Log                 X: ${x}                                       # X: 559.09375
+        | # Assertions:
+        | Get BoundingBox     id=element         width         >    180
+        | Get BoundingBox     id=element         ALL           validate    value['x'] > value['y']*2
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.GetBoundingBox(Request.ElementSelector(selector=selector))
-            parsed = json.loads(response.body)
+            parsed = json.loads(response.json)
             logger.debug(parsed)
-            if keys:
-                parsed = {key.name: parsed[key.name] for key in keys}
-            return parsed
+            if key == BoundingBoxFields.ALL:
+                return int_dict_verify_assertion(
+                    parsed, assertion_operator, assertion_expected, "BoundingBox is"
+                )
+            else:
+                logger.info(f"Value of '{key}'': {parsed[key.name]}")
+                return float_str_verify_assertion(
+                    parsed[key.name],
+                    assertion_operator,
+                    assertion_expected,
+                    f"{key} is ",
+                )
