@@ -1,3 +1,17 @@
+# Copyright 2020-     Robot Framework Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import json
 from pathlib import Path
 from typing import Dict
@@ -25,7 +39,9 @@ class Waiter(LibraryComponent):
 
         If the selector doesn't satisfy the condition within the timeout the keyword will FAIL.
 
-        ``state``
+        ``selector`` <str> Selector of the corresponding object. **Required**
+
+        ``state`` Defaults to visible. Possible states are:
         - ``attached``: to be present in DOM.
         - ``detached``: to not be present in DOM.
         - ``visible``: to have non-empty bounding box and no visibility:hidden.
@@ -44,23 +60,28 @@ class Waiter(LibraryComponent):
         Note that element without any content or with display:none has an empty bounding box
         and is not considered visible.
 
-        ``timeout``: (optional) uses default timeout if not set.
+        ``timeout`` <str> (optional) uses default timeout of 10 seconds if not set.
         """
         funct = {
-            "enabled": "e => !e.disabled",
-            "disabled": "e => e.disabled",
-            "editable": "e => !e.readOnly",
-            "readonly": "e => e.readOnly",
-            "selected": "e => e.selected",
-            "deselected": "e => !e.selected",
-            "focused": "e => document.activeElement === e",
-            "defocused": "e => document.activeElement !== e",
-            "checked": "e => e.checked",
-            "unchecked": "e => !e.checked",
+            ElementState.enabled: "e => !e.disabled",
+            ElementState.disabled: "e => e.disabled",
+            ElementState.editable: "e => !e.readOnly",
+            ElementState.readonly: "e => e.readOnly",
+            ElementState.selected: "e => e.selected",
+            ElementState.deselected: "e => !e.selected",
+            ElementState.focused: "e => document.activeElement === e",
+            ElementState.defocused: "e => document.activeElement !== e",
+            ElementState.checked: "e => e.checked",
+            ElementState.unchecked: "e => !e.checked",
         }
 
         with self.playwright.grpc_channel() as stub:
-            if state.name in ["attached", "detached", "visible", "hidden"]:
+            if state in [
+                ElementState.attached,
+                ElementState.detached,
+                ElementState.visible,
+                ElementState.hidden,
+            ]:
                 options: Dict[str, object] = {"state": state.name}
                 if timeout:
                     timeout_ms = timestr_to_millisecs(timeout)
@@ -72,10 +93,8 @@ class Waiter(LibraryComponent):
                     )
                 )
                 logger.info(response.log)
-            elif state.name in funct:
-                self.wait_for_function(
-                    funct[state.name], selector=selector, timeout=timeout
-                )
+            else:
+                self.wait_for_function(funct[state], selector=selector, timeout=timeout)
 
     @keyword(tags=["Wait", "PageContent"])
     def wait_for_function(
@@ -88,19 +107,22 @@ class Waiter(LibraryComponent):
         """Polls JavaScript expression or function in browser until it returns a
         (JavaScript) truthy value.
 
-        ``function`` a valid javascript function or a javascript function body. For example
-        ``() => true`` and ``true`` will behave similarly.
+        ``function`` <str> A valid javascript function or a javascript function body. For example
+        ``() => true`` and ``true`` will behave similarly. **Required**
 
-        ``selector`` Selector to resolve and pass to the JavaScript function. This will be the first
+        ``selector`` <str> Selector to resolve and pass to the JavaScript function. This will be the first
         argument the function receives. If given a selector a function is necessary, with an argument
         to capture the elementhandle. For example ``(element) => document.activeElement === element``
 
-        Default polling value of "raf" polls in a callback for ``requestAnimationFrame``.
+        ``polling`` <str> Default polling value of "raf" polls in a callback for ``requestAnimationFrame``.
         Any other value for polling will be parsed as a robot framework time for interval between polls.
 
-        ``timeout``: (optional) uses default timeout if not set.
+        ``timeout`` <str> Uses default timeout of 10 seconds if not set.
 
-        [https://github.com/MarketSquare/robotframework-browser/tree/master/atest/test/06_Examples/js_evaluation.robot | Usage examples. ]
+        Example usage:
+        | ${promise}    Promise To      Wait For Function    element => element.style.width=="100%"    selector=\\#progress_bar    timeout=4s
+        | Click         \\#progress_bar
+        | Wait For      ${promise}
         """
         with self.playwright.grpc_channel() as stub:
             options: Dict[str, int] = {}
@@ -114,32 +136,34 @@ class Waiter(LibraryComponent):
                     script=function, selector=selector, options=options_json,
                 )
             )
+            logger.debug(response.json)
             logger.info(response.log)
 
-    @keyword(tags=["Wait"])
+    @keyword(tags=["Wait", "BrowserControl"])
     def wait_for_download(self, saveAs: str = ""):
-        """ Waits for next download event on page. Returns file path to downloaded file.
+        """Waits for next download event on page. Returns file path to downloaded file.
 
-            To enable downloads context's ``acceptDownloads`` needs to be true.
+        To enable downloads context's ``acceptDownloads`` needs to be true.
 
-            With default filepath downloaded files are deleted when Context the download happened in is closed.
+        With default filepath downloaded files are deleted when Context the download happened in is closed.
 
-            ``saveAs`` filename to save as. File will also temporarily be saved in playwright context's default DL location.
+        ``saveAs`` <str> Filename to save as. File will also temporarily be saved in playwright context's default download location.
 
-            | New Context    | acceptDownloads=True
-            | New Page       | ${LOGIN_URL}
-            | ${dl_promise}  | Promise To  Wait For Download
-            | Click          | \\#file_download
-            | ${file_path}=  | Wait For  ${dl_promise}
+        Example usage:
+        | New Context      acceptDownloads=True
+        | New Page         ${LOGIN_URL}
+        | ${dl_promise}    Promise To  Wait For Download
+        | Click            \\#file_download
+        | ${file_path}=    Wait For  ${dl_promise}
         """
         with self.playwright.grpc_channel() as stub:
             if not saveAs:
                 response = stub.WaitForDownload(Request().FilePath())
                 logger.debug(response.log)
-                return json.loads(response.body)
+                return json.loads(response.json)
             else:
                 p = Path(saveAs)
                 p.resolve()
                 response = stub.WaitForDownload(Request().FilePath(path=str(p)))
                 logger.debug(response.log)
-                return json.loads(response.body)
+                return json.loads(response.json)
